@@ -22,7 +22,7 @@ Net::SSLeay::PEM class in L<SPVM> represents OpenSSL PEM data structure.
 
 C<static method read_bio_X509 : L<Net::SSLeay::X509|SPVM::Net::SSLeay::X509> ($bp : L<Net::SSLeay::BIO|SPVM::Net::SSLeay::BIO>);>
 
-Calls L<PEM_read_bio_X509|https://docs.openssl.org/3.0/man3/PEM_read_bio_PrivateKey/> function, creates a new L<Net::SSLeay::X509|SPVM::Net::SSLeay::X509> object, sets the pointer value of the new object to the return value of L<PEM_read_bio_X509|https://docs.openssl.org/3.0/man3/PEM_read_bio_PrivateKey/> function, and returns the new object.
+Calls native L<PEM_read_bio_X509|https://docs.openssl.org/3.0/man3/PEM_read_bio_PrivateKey/> function, creates a new L<Net::SSLeay::X509|SPVM::Net::SSLeay::X509> object, sets the pointer value of the new object to the return value of the native function, and returns the new object.
 
 Exceptions:
 
@@ -34,7 +34,7 @@ If PEM_read_bio_X509 failed, an exception is thrown with C<eval_error_id> set to
 
 C<static method read_bio_X509_CRL : L<Net::SSLeay::X509_CRL|SPVM::Net::SSLeay::X509_CRL> ($bp : L<Net::SSLeay::BIO|SPVM::Net::SSLeay::BIO>);>
 
-Calls L<read_bio_X509_CRL|https://docs.openssl.org/3.0/man3/PEM_read_bio_PrivateKey/> function, creates a new L<Net::SSLeay::X509_CRL|SPVM::Net::SSLeay::X509_CRL> object, sets the pointer value of the new object to the return value of L<read_bio_X509_CRL|https://docs.openssl.org/3.0/man3/PEM_read_bio_PrivateKey/> function, and returns the new object.
+Calls native L<read_bio_X509_CRL|https://docs.openssl.org/3.0/man3/PEM_read_bio_PrivateKey/> function, creates a new L<Net::SSLeay::X509_CRL|SPVM::Net::SSLeay::X509_CRL> object, sets the pointer value of the new object to the return value of the native function, and returns the new object.
 
 Exceptions:
 
@@ -42,6 +42,18 @@ The BIO $bp must be defined. Otherwise an exception is thrown.
 
 If PEM_read_bio_X509_CRL failed, an exception is thrown with C<eval_error_id> set to the basic type ID of L<Net::SSLeay::Error|SPVM::Net::SSLeay::Error> class.
 
+=head2 read_bio_DHparams
+
+C<static method read_bio_DHparams : L<Net::SSLeay::DH|SPVM::Net::SSLeay::DH> ($bp : L<Net::SSLeay::BIO|SPVM::Net::SSLeay::BIO>);>
+
+Calls native L<read_bio_DHparams|https://docs.openssl.org/3.0/man3/read_bio_DHparams/> function, creates a new L<Net::SSLeay::X509_CRL|SPVM::Net::SSLeay::X509_CRL> object, sets the pointer value of the new object to the return value of the native function, and returns the new object.
+
+Exceptions:
+
+The BIO $bp must be defined. Otherwise an exception is thrown.
+
+If read_bio_DHparams failed, an exception is thrown with C<eval_error_id> set to the basic type ID of L<Net::SSLeay::Error|SPVM::Net::SSLeay::Error> class.
+
 =head1 See Also
 
 =over 2
